@@ -291,22 +291,27 @@ async def create_validator_proposal(payer_keypair: KeypairInput, proposal_numera
 
 async def vote_validator_proposal(payer_keypair: KeypairInput, proposal_numeration: int, mint_pubkeys: List[PubkeyInput], val_index:int, client: AsyncClient, log_level: int = 0) -> str:
     proposal_pubkey, _proposal_bump = PublicKey.find_program_address([bytes(ingl_constants.PROPOSAL_KEY, 'UTF-8'), proposal_numeration.to_bytes(4,"big")], ingl_constants.INGL_PROGRAM_ID)
-
+    global_gem_pubkey, _global_gem_bump = PublicKey.find_program_address([bytes(ingl_constants.GLOBAL_GEM_KEY, 'UTF-8')], ingl_constants.INGL_PROGRAM_ID)
 
     
     payer_account_meta = AccountMeta(payer_keypair.public_key, True, True)
     proposal_meta = AccountMeta(proposal_pubkey, False, True)
+    global_gem_meta = AccountMeta(global_gem_pubkey, False, True)
+    system_program_meta = AccountMeta(system_program.SYS_PROGRAM_ID, False, False)
 
     accounts = [
         payer_account_meta,
-        proposal_meta
+        proposal_meta,
+        global_gem_meta,
         ]
 
     for mint in mint_pubkeys:
         gem_account_pubkey, _ = PublicKey.find_program_address([bytes(ingl_constants.GEM_ACCOUNT_CONST, 'UTF-8'), bytes(mint.public_key)], ingl_constants.INGL_PROGRAM_ID)
         accounts.append(AccountMeta(mint.public_key, False, False))
         accounts.append(AccountMeta(assoc_instructions.get_associated_token_address(payer_keypair.public_key, mint.public_key), False, False))
-        accounts.append(AccountMeta(gem_account_pubkey, False, True) )
+        accounts.append(AccountMeta(gem_account_pubkey, False, True))
+    
+    accounts.append(system_program_meta)
 
 
 
@@ -427,7 +432,6 @@ async def undelegate_nft(payer_keypair: KeypairInput, mint_pubkey: PubkeyInput,
         gem_account_meta,
         mint_associated_meta,
         global_gem_meta,
-        validator_account_meta,
         system_program_meta,
         authorized_withdrawer_meta,
 
